<?php

//
//  Name:        nestInterface.php
//  Author:      Andrew Seyes
//  Version      1.0 - Initial Release 
//  Description: Provide a Nest Interface via the Unofficial Nest Learning Thermostat API (https://github.com/gboudreau/nest-api). 
//  Arguments:      argv[1]   	 	argv[2]		argv[3]		argv[4]
//               1) Encrypt        	<token>		<password>
//               2) EndAway        	<token>		(nestSerial)
//               3) getDeviceInfo  	<token>		(nestSerial)
//               4) SetAway        	<token>		(nestSerial)
//               5) SetTemperature 	<token>		<temperature> 	(nestSerial)
//                  <..> = required parameter
//                  (..) = optional parameter

$version = '1.0';

try {
     require 'nestInterface.cfg';
     require 'nestInterfaceFunctions.php';
     require 'nest-api-master/nest.class.php';

     if (count($argv) < 2) throw new Exception('Too few parameters');

     $script    = $argv[0];
     $command   = $argv[1];
     $token     = $argv[2];

     define('TOKEN', $token);

     if (strlen($token) != 64) throw new Exception('Invalid token length');

     switch ("$command") {
	case "Encrypt":
     		printf("Script Arguments: %s %s\n",$script,$command);
		$encrypted = setNestPass($argv[3]);
		echo "Result: $encrypted\n";
		break;

	case "EndAway":
		$nestSerial = (empty($argv[3]) ? "" : $argv[3]);		
		printf("Script Arguments: %s %s %s\n",$script,$command,$nestSerial);

     		define('USERNAME', $config['nest_user']);
     		define('PASSWORD', getNestPass($config['nest_pass']));

		// Connect to Nest and End Away
		$nest   = new Nest();
		$success = $nest->setAway(AWAY_MODE_OFF,$nestSerial); 
     		var_dump($success);
		if (!$success) throw new Exception('Unable to end Away');
		break;

	case "getDeviceInfo":
		$nestSerial = (empty($argv[3]) ? "" : $argv[3]);
		printf("Script Arguments: %s %s %s\n",$script,$command,$nestSerial);

     		define('USERNAME', $config['nest_user']);
     		define('PASSWORD', getNestPass($config['nest_pass']));

		// Connect to Nest and retrieve device information
		$nest       = new Nest();
	        $deviceInfo = $nest->getDeviceInfo($nestSerial);

	        // Get device information
	        $autoAway           = $deviceInfo->current_state->auto_away;
	        $battery            = $deviceInfo->current_state->battery_level;
	        $currentTemperature = $deviceInfo->current_state->temperature;
	        $heat               = $deviceInfo->current_state->heat;
	        $humidity           = $deviceInfo->current_state->humidity;
	        $lastConnection     = $deviceInfo->network->last_connection; 
	        $leaf               = $deviceInfo->current_state->leaf;
	        $localIP            = $deviceInfo->network->local_ip;
	        $location           = $deviceInfo->where;
	        $manualAway         = $deviceInfo->current_state->manual_away;
	        $mode               = $deviceInfo->current_state->mode;
	        $online             = $deviceInfo->network->online;
	        $serialNumber       = $deviceInfo->serial_number;;
	        $targetTemperature  = (preg_match("/away/", $mode) ?  $deviceInfo->target->temperature[0] : $deviceInfo->target->temperature);	
	        $temperatureScale   = $deviceInfo->scale;
	        $timeToTarget       = $deviceInfo->target->time_to_target;
	        $wanIP              = $deviceInfo->network->wan_ip;

     		// lastConnection timestamp format: yyyy/mm/dd - hh:mm:ss
     		// We display this as "yyyy-mm-dd hh:mm:ssZ" for Homegenie!
     		$day   = substr($lastConnection,8,2);
     		$month = substr($lastConnection,5,2);
     		$year  = substr($lastConnection,0,4);
     		$time  = substr($lastConnection,11,5);

     		// Display information
     		printf("Auto.Away           = %s\n",           $autoAway);
     		printf("Battery             = %.02f\n",        $battery);
     		printf("Current.Temperature = %.01f\n",        $currentTemperature);
     		printf("Heating             = %s\n",           $heat);
     		printf("Current.Humidity    = %s\n",           $humidity);
     		printf("Last.Connection     = %s-%s-%s %sZ\n", $year,$month,$day,$time);
     		printf("Leaf                = %s\n",           $leaf);
     		printf("Local.IP            = %s\n",           $localIP);
     		printf("Location            = %s\n",           $location);
     		printf("Manual.Away         = %s\n",           $manualAway);
     		printf("Mode                = %s\n",           $mode);
     		printf("Online              = %s\n",           $online);
     		printf("Serial.Number       = %s\n",           $serialNumber);
     		printf("Target.Temperature  = %.0f\n",         $targetTemperature);
     		printf("Temperature.Scale   = %s\n",           $temperatureScale);
     		printf("Time.To.Target      = %s\n",           $timeToTarget);
     		printf("Wan.IP              = %s\n",           $wanIP);

		break;

	case "SetAway":
		$nestSerial = (empty($argv[3]) ? "" : $argv[3]);
		printf("Script Arguments: %s %s %s\n",$script,$command,$nestSerial);

     		define('USERNAME', $config['nest_user']);
     		define('PASSWORD', getNestPass($config['nest_pass']));

		// Connect to Nest and Set Away
		$nest    = new Nest();
		$success = $nest->setAway(AWAY_MODE_ON,$nestSerial); 
     		var_dump($success);
		if (!$success) throw new Exception('Unable to set Away');
		break;

	case "SetTemperature":
		$targettemp = (int)$argv[3];
		$nestSerial = (empty($argv[4]) ? "" : $argv[4]);
		printf("Script Arguments: %s %s %s %s\n",$script,$command,$targettemp,$nestSerial);

     		define('USERNAME', $config['nest_user']);
     		define('PASSWORD', getNestPass($config['nest_pass']));

     		// Note: setting temperatures will use the units you set on the device. 
     		echo "Setting target temperature to $targettemp\n";
     		$nest    = new Nest();
     		$success = $nest->setTargetTemperature($targettemp,$nestSerial);
     		var_dump($success);
		if (!$success) throw new Exception('Unable to set temperature');
		break;

	default: 
     		echo "Unknown command: $command\n";
     }

}

// Catch any exception
catch(Exception $e) {
    printf("SCRIPT-ERROR: %s\n", $e->getMessage());
}

?>
