<?php

//
//  Name:        nestInterfaceFunctions.php
//  Version      1.0 - Initial Release
//  Author:	 http://stackoverflow.com/questions/9262109/php-simplest-two-way-encryption
//  Description: Provide encryption / decryption functions for the nestInterface.php script

define("HASH_ALGORITHM","sha256");
define("CIPHER_METHOD","aes-256-ctr");

function getNestPass($encryptedb64) 
{
     	$binaryToken = hex2bin(TOKEN);
        $cryptoKey   = hash_hmac(HASH_ALGORITHM, 'ENCRYPTION',     $binaryToken, true);
        $authKey     = hash_hmac(HASH_ALGORITHM, 'AUTHENTICATION', $binaryToken, true);

        $encrypted = base64_decode($encryptedb64, true);
        if ($encrypted === false) throw new Exception('Decryption failure - base64 decode of message failed');

	// Verification of Message Authentication Code
        $hashsize      = mb_strlen(hash(HASH_ALGORITHM, '', true), '8bit');
        $msgAuthCode   = mb_substr($encrypted, 0, $hashsize, '8bit');
        $ciphertextb64 = mb_substr($encrypted, $hashsize, null, '8bit');
        $calculatedMac = hash_hmac(HASH_ALGORITHM,$ciphertextb64,$authKey,true);
        $initVector    = openssl_random_pseudo_bytes(32);

        if (hash_hmac(HASH_ALGORITHM, $msgAuthCode, $initVector) === hash_hmac(HASH_ALGORITHM, $calculatedMac, $initVector)) {
	    // Verification is OK
	}	
	else throw new Exception('Decryption failure - MAC verification failed');

	// Decrypt ciphertext
        $ciphertext = base64_decode($ciphertextb64, true);
        if ($ciphertext === false) throw new Exception('Decryption failure - base64 decode of ciphertext failed');

        $initVectorLength = openssl_cipher_iv_length(CIPHER_METHOD);
        $initVector       = mb_substr($ciphertext, 0, $initVectorLength, '8bit');
        $ciphertext       = mb_substr($ciphertext, $initVectorLength, null, '8bit');
        $plaintext        = openssl_decrypt($ciphertext,CIPHER_METHOD,$cryptoKey,OPENSSL_RAW_DATA,$initVector);

        return $plaintext;
}

function setNestPass($plaintext) 
{
     	$binaryToken      = hex2bin(TOKEN);
        $cryptoKey        = hash_hmac(HASH_ALGORITHM, 'ENCRYPTION',     $binaryToken, true);
        $authKey          = hash_hmac(HASH_ALGORITHM, 'AUTHENTICATION', $binaryToken, true);

        $initVectorLength = openssl_cipher_iv_length(CIPHER_METHOD);
        $initVector       = openssl_random_pseudo_bytes($initVectorLength);
        $ciphertext       = openssl_encrypt($plaintext,CIPHER_METHOD,$cryptoKey,OPENSSL_RAW_DATA,$initVector);

        $ciphertextb64    = base64_encode($initVector.$ciphertext);
        $msgAuthCode      = hash_hmac(HASH_ALGORITHM, $ciphertextb64, $authKey, true);

        return base64_encode($msgAuthCode.$ciphertextb64);
}

?>
